import time
import json
import logging
from datetime import datetime

import pytz

try:
    import redis
except ImportError:
    pass


class RedisHandler(logging.Handler):
    """
    Publish messages to redis channel.
    """

    def __init__(self, channel, redis_url):
        """
        Create a new logger for the given channel and redis_url
        """
        logging.Handler.__init__(self)
        self.channel = channel
        self.redis_client = redis.from_url(redis_url)

    def emit(self, record):
        """
        Publish record to redis logging channel
        """
        try:
            self.redis_client.publish(self.channel, self._format_record(record))
        except redis.RedisError:
            pass
        except Exception:
            self.handleError(record)

    def _format_record(self, record):
        # format log into tuple of:
        # (datetime, progress, message)

        # Get a timezone aware datetime from time.time()
        dt = datetime.fromtimestamp(record.created).replace(tzinfo=pytz.timezone(time.strftime("%Z", time.gmtime()))).astimezone(pytz.utc)
        payload = { "time"    : dt.isoformat(),
                    "value"   : None,
                    "message" : None,
                    "level"   : record.levelname }

        msg = record.msg

        """ This allows for things like:

            logger.progress((2, "Initializing particles"))
            logger.info("word to yo momma")
            logger.progress(30)
        """
        if isinstance(msg, list) or isinstance(msg, tuple):
            payload["value"] = msg[0]
            payload["message"] = msg[1]
        elif isinstance(msg, str) or isinstance(msg, unicode):
            payload["message"] = msg
        elif isinstance(msg, float) or isinstance(msg, int):
            payload["value"] = msg
        else:
            payload["message"] = msg

        return json.dumps(payload)

    def close(self):
        self.redis_client.close()
        logging.Handler.close(self)



from __future__ import with_statement

from setuptools import setup, find_packages

from paegan import __version__


def readme():
    with open('README.md') as f:
        return f.read()

reqs = [line.strip() for line in open('requirements.txt')]

setup(
    namespace_packages = ['paegan'],
    name                 = "paegan",
    version              = __version__,
    description          = "Processing and Analysis for Numerical Data",
    long_description     = readme(),
    license              = 'GPLv3',
    author               = "Kyle Wilcox",
    author_email         = "kyle@axiomdatascience.com",
    url                  = "https://github.com/axiom-data-science/paegan",
    packages             = find_packages(),
    install_requires     = reqs,
    classifiers          = [
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python',
            'Topic :: Scientific/Engineering',
        ],
    include_package_data = True,
) 
